
// Gives us access to all functions in the 'system' namespace
using System;


class InheritFromMe {

	// a property will map to this value
	private int a_value;
	
	// constructor
	public InheritFromMe(){
		Console.Write("\nBase constructor called");
	}
	
	// declare a property for 'a_value'
	public int A_Value
	{
		get
		{
			return a_value;
		}
		set
		{
			a_value = value;
		}
	}
	
	// A virtual function which can be overriden
	// by a derived class
	public virtual void VirtualFunction(){
		Console.Write("\nThis is base VirtualFunction 1");
	}
	
	public virtual void VirtualFunction2(){
		Console.Write("\nThis is base VirtualFunction 2");
	}
	
	// A normal method
	public void NormalMethod(){
		Console.Write("\nThis is base NormalMethod");
	}
}

class Inherits : InheritFromMe {

	// Constructor
	public Inherits(){
		Console.Write("\nDerived constructor called");
	}

	// Add a new method in addition to the two inherited
	// This method demonstrates how we can force a call
	// to a virtual function in an object's base class
	public void demonstrateBaseCall(){
		base.VirtualFunction();
	}
	
	// Now we override just one virtual method and comment out
	// what would have been the code to override the second.
	// ------------------------------------------------------>
	
	public override void VirtualFunction(){
		Console.Write("\nThis is derived VirtualFunction 1");
	}
	
	//public override void VirtualFunction2(){
		//Console.Write("\nThis is derived VirtualFunction 2");
	//}
	
	// <------------------------------------------------------

	// Finally we try and implement a new version of 'NormalMethod'
	// by a process known as HIDING which can only happen when
	// the base class doesn't declare its version of the function
	// as virtual:
	
	public new void NormalMethod(){
		Console.Write("\nThis is derived NormalMethod");
	}
}


class GettingStuckIntoClasses2{
		
	public static int Main(){	
	
		// declare an instance of 'InheritFromMe' class, and
		// an instance of 'Inherits'.		
		InheritFromMe baseInstance = new InheritFromMe();
		Inherits derivedClass = new Inherits();

		// Now call both object's VirtualFunction methods
		// to prove that the derived class has overriden
		// the base class implementation
		// We also demonstrate that where the derived class
		// chose not to override the VirtualFunction2, C#
		// is smart enough to call the base class implementation
		// as there is no derived implementation
		baseInstance.VirtualFunction();
		derivedClass.VirtualFunction();
		baseInstance.VirtualFunction2();
		derivedClass.VirtualFunction2();
			
		// Now call the derived class's extra method to
		// demonstrate how calls can be forced to the
		// base class implementation
		derivedClass.demonstrateBaseCall();
		
		// Now create a new reference of type
		// 'InheritFromMe' and point it at a new
		// instance of 'Inherits' then call
		// a virtual function.  This is
		// Polymorphism in action
		InheritFromMe baseReference = new Inherits();
		baseReference.VirtualFunction();

		// On a final note, consider the function
		// 'NormalMethod'.  We couldn't override
		// it, so we 'hid' it in 'class Inherits'.  
		// Now if we access two objects of type
		// 'Inherits', but once through a 'InheritFromMe'
		// reference and once through a 'Inherits'
		// reference, we'll see that where polymorphism
		// would normally take place, we're instead
		// not achieving it.
		derivedClass.NormalMethod();
		baseReference.NormalMethod(); // we want this to say
									  // 'derived' because it
									  // points to a derived object
									  // but it will say 'base'
			
		return 1; // any int value will do
	}
}
